/*
 * Copyright (c) 2015, 2016, 2017, 2018, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *
 *     * Neither the name of Intel Corporation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY LOG OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GEOPM_SCHED_H_INCLUDE
#define GEOPM_SCHED_H_INCLUDE

#include <stdio.h>

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __linux__
/// @brief cpuset definition for non-linux platforms.
typedef struct cpu_set_t {
    long int x[512];
} cpu_set_t;

static inline void CPU_SET(int cpu, cpu_set_t *set)
{
    int array_num = -1;
    long comp_mask;

    array_num = cpu / 64;
    comp_mask = 1 << (cpu % 64);

    set->x[array_num] |= comp_mask;
}

static inline int  CPU_ISSET(int cpu, cpu_set_t *set)
{
    int array_num = -1;
    long comp_mask;

    array_num = cpu / 64;
    comp_mask = 1 << (cpu % 64);

    return set->x[array_num] &= comp_mask;
}
#else
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <sched.h>
#endif

int geopm_sched_num_cpu(void);

int geopm_sched_get_cpu(void);

int geopm_sched_proc_cpuset(int num_cpu, cpu_set_t *proc_cpuset);

int geopm_sched_woomp(int num_cpu, cpu_set_t *woomp);

int geopm_sched_popen(const char *cmd, FILE **fid);

#ifdef __cplusplus
}
#endif

#endif
